@@ -33,9 +33,10 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
 #include <netdb.h>
 #include <errno.h>
-#include <malloc.h>
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -87,6 +88,11 @@ struct oem_handler {
     void (*init)(struct msg_info *mi);
 };
 
+#define EVENT_BUFFER_GLOBAL_ENABLE	(1 << 2)
+#define EVENT_LOG_GLOBAL_ENABLE		(1 << 3)
+#define SUPPORTED_GLOBAL_ENABLES	(EVENT_BUFFER_GLOBAL_ENABLE | \
+					 EVENT_LOG_GLOBAL_ENABLE)
+
 struct msg_info {
     /* General info, set by main code, not formatters. */
     void          *info;
@@ -106,7 +112,8 @@ struct msg_info {
     int           debug;
     int           do_attn;
     unsigned char attn_chars[8];
-    int           attn_chars_len;
+    unsigned int  attn_chars_len;
+    char          global_enables;
 
     /* Info from the recv message. */
     unsigned char netfn;
@@ -245,8 +252,8 @@ struct ra_data {
 static void ra_format_msg(const unsigned char *msg, unsigned int msg_len,
 			  struct msg_info *mi)
 {
-    int i;
-    int len;
+    unsigned int i;
+    unsigned int len;
     unsigned char c[RA_MAX_CHARS_SIZE];
 
     len = 0;
@@ -564,8 +571,8 @@ struct tm_data {
 static void tm_format_msg(const unsigned char *msg, unsigned int msg_len,
 			  struct msg_info *mi)
 {
-    int i;
-    int len;
+    unsigned int i;
+    unsigned int len;
     unsigned char c[TM_MAX_CHARS_SIZE];
     unsigned char t;
 
@@ -766,7 +773,7 @@ static void
 socket_send(const unsigned char *data, unsigned int len, struct msg_info *mi)
 {
     int rv;
-    int i;
+    unsigned int i;
 
     if (mi->debug > 0) {
 	printf("Sock send:");
@@ -783,7 +790,7 @@ socket_send(const unsigned char *data, unsigned int len, struct msg_info *mi)
     if (rv < 0) {
 	perror("write");
 	return;
-    } else if (rv < len) {
+    } else if (((unsigned int) rv) < len) {
 	len -= rv;
 	data += rv;
 	goto restart;
@@ -793,6 +800,8 @@ socket_send(const unsigned char *data, unsigned int len, struct msg_info *mi)
 #define IPMI_APP_NETFN	6
 #define IPMI_GET_DEV_ID_CMD	0x01
 #define IPMI_GET_DEVICE_GUID_CMD 0x08
+#define IPMI_SET_BMC_GLOBAL_ENABLES_CMD	0x2e
+#define IPMI_GET_BMC_GLOBAL_ENABLES_CMD	0x2f
 #define IPMI_GET_MSG_FLAGS_CMD	0x31
 #define IPMI_GET_MSG_CMD	0x33
 #define IPMI_SEND_MSG_CMD	0x34
@@ -811,7 +820,7 @@ handle_ipmb_msg(const unsigned char *msg, unsigned int len,
     struct msg *imsg;
     unsigned char rsp[IPMI_MAX_MSG_LENGTH];
     unsigned int rsp_len;
-    int          i;
+    unsigned int i;
 
     imsg = malloc(sizeof(*imsg));
     if (!imsg)
@@ -866,7 +875,7 @@ static unsigned char guid_data[] = {
 static void
 handle_msg(const unsigned char *msg, unsigned int len, struct msg_info *mi)
 {
-    int i;
+    unsigned int i;
     unsigned char rsp[IPMI_MAX_MSG_LENGTH];
     unsigned int rsp_len;
     struct msg *m;
@@ -907,6 +916,8 @@ handle_msg(const unsigned char *msg, unsigned int len, struct msg_info *mi)
 	case IPMI_GET_MSG_FLAGS_CMD:
 	    rsp[0] = 0;
 	    rsp[1] = 0;
+	    if (mi->event_q)
+		rsp[1] |= 2;
 	    if (mi->ipmb_q)
 		rsp[1] |= 1;
 	    rsp_len = 2;
@@ -957,6 +968,31 @@ handle_msg(const unsigned char *msg, unsigned int len, struct msg_info *mi)
 	    rsp_len = 1;
 	    break;
 
+	case IPMI_SET_BMC_GLOBAL_ENABLES_CMD:
+	    if (len < 1) {
+		rsp[0] = 0xcc;
+		rsp_len = 1;
+		break;
+	    }
+
+	    if ((msg[0] & ~SUPPORTED_GLOBAL_ENABLES) != 0) {
+		rsp[0] = 0xcc;
+		rsp_len = 1;
+		break;
+	    }
+
+	    mi->global_enables = msg[0];
+
+	    rsp[0] = 0;
+	    rsp_len = 1;
+	    break;
+
+	case IPMI_GET_BMC_GLOBAL_ENABLES_CMD:
+	    rsp[0] = 0;
+	    rsp[1] = mi->global_enables;
+	    rsp_len = 2;
+	    break;
+
 	case IPMI_READ_EVENT_MSG_CMD:
 	    if (!mi->event_q) {
 		rsp[0] = 0x80;
@@ -1290,7 +1326,7 @@ usage(void)
 int
 main(int argc, char *argv[])
 {
-    int i;
+    unsigned int i;
     struct addrinfo hints, *res0;
     sockaddr_ip_t addr;
     struct msg_info *mi = &main_mi;
@@ -1378,7 +1414,7 @@ main(int argc, char *argv[])
     }
 
     i = optind;
-    if (i+2 < argc) {
+    if (i+2 < ((unsigned int) argc)) {
 	fprintf(stderr, "Host and/or port not supplied\n");
 	usage();
     }
@@ -1406,6 +1442,13 @@ main(int argc, char *argv[])
 	usage();
     }
 
+    i = 1;
+    if (setsockopt(mi->sock, IPPROTO_TCP, TCP_NODELAY,
+		   (char *) &i, sizeof(i)) == -1) {
+	perror("setsockopt TCP_NODELAY");
+	usage();
+    }
+
     mi->info = mi->codec->setup();
     if (!mi->info) {
 	fprintf(stderr, "Out of memory\n");
@@ -1428,6 +1471,7 @@ main(int argc, char *argv[])
 	unsigned char buf[128];
 	int i;
 	fd_set readfds;
+	int rv2;
 
 	FD_ZERO(&readfds);
 	FD_SET(0, &readfds);
@@ -1447,13 +1491,30 @@ main(int argc, char *argv[])
 		perror("read");
 		usage();
 	    }
+
+	    if (mi->debug > 1) {
+		printf("recv:");
+		for (i=0; i<rv; i++) {
+		    if ((i % 16) == 0)
+			printf("\n  ");
+		    printf(" %2.2x(%c)", buf[i],
+			   isprint(buf[i]) ? buf[i] : ' ');
+		}
+		printf("\n");
+	    }
+
 	    for (i=0; i<rv; i++) {
 		/*
 		 * Echo one at a time in case the echo gets turned off
 		 * in the middle of this data.
 		 */
-		if (mi->echo)
-		    write(mi->sock, buf+i, 1);
+		if (mi->echo) {
+		    rv2 = write(mi->sock, buf+i, 1);
+		    if (rv2 < 0) {
+			perror("write");
+			usage();
+		    }
+		}
 		mi->codec->handle_char(buf[i], mi);
 	    }
 	}
